package cn.iocoder.yudao.module.system.service.robot;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.module.system.constant.robot.RobotTaskChcheConstant;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.houselocation.WareHouseLocationMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotInformationMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskDetailMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskMapper;
import cn.iocoder.yudao.module.system.enums.redis.RobotCacheLockEnum;
import cn.iocoder.yudao.module.system.enums.robot.*;
import cn.iocoder.yudao.module.system.util.redis.RedisUtil;
import cn.iocoder.yudao.module.system.util.redis.RedissonUtils;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception0;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;
import static org.apache.groovy.util.concurrent.concurrentlinkedhashmap.Weighers.map;

/**
 * 机器人任务主表 Service 实现类
 *
 * @author 陈宾顺
 */
@Slf4j
@Service
@Validated
public class RobotTaskServiceImpl implements RobotTaskService {

    private final String RESULT = "SUCCESS";

    @Resource
    private RobotTaskMapper taskMapper;

    @Resource
    private WareHouseLocationMapper locationMapper;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private RedissonUtils redissonUtils;

    @Resource
    private RobotTaskDetailMapper taskDetailMapper;

    @Resource
    private RobotTaskDetailService robotTaskDetailService;

    @Resource
    private RobotInformationMapper informationMapper;

    @Value("${zn.task-no:ZN}")
    private String taskNo;
    @Value("${zn.do_cycle:true}")
    private Boolean doCycle;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createTask(RobotTaskSaveReqVO createReqVO) throws InterruptedException {
        //校验
        doTaskCheck(createReqVO);

        //设置任务号
        if (ObjectUtil.isEmpty(createReqVO.getTaskNo())) {
            String incrementByKey = redisUtil.getIncrementByKey(RobotCacheLockEnum.TASK_NO.getKey());
            createReqVO.setTaskNo(taskNo+ DateUtils.getYearMonthDay()+incrementByKey);
        }
        //获取库位id
        RLock lock = redissonUtils.getLock(RobotCacheLockEnum.ROBOT_TASK_ADD_LOCK.getKey());

        String addResult = "";
        if (lock.tryLock(30l,TimeUnit.MINUTES)){
            try {
                addResult = addTask(createReqVO);
            } catch (Exception e) {
                log.error("下发任务出现异常 :{}",e);
                addResult = e.getMessage();
            } finally {
                lock.unlock();
            }
        }else {
            throw exception(REDISSON_NOT_OBTAIN_LOCK);
        }

        if (!RESULT.equals(addResult)) {
            throw exception0(TASK_CHECK_EXCEPTION.getCode(), addResult);
        }

        return 1l;
    }

    /**
     * 校验
     * @param createReqVO
     */
    private void doTaskCheck(RobotTaskSaveReqVO createReqVO) {
        //校验明细
        if (ObjectUtil.isEmpty(createReqVO.getTaskDetailList())) {
            throw exception(TASK_DATA_NOT_FULL);
        }
        //校验循环任务的配置未开启
        if (DoCycleEnum.YES.getType().equals(createReqVO.getDoCycle()) && !doCycle) {
            throw exception(TASK_CYCLE_NOT_OPEN);
        }
        //搬空任务只能选择线库或者区域
        if (MoveAllEnum.YES.getType().equals(createReqVO.getDoMoveAll())) {
            Set<Integer> releaseTypeList =
                    createReqVO.getTaskDetailList().stream().map(v -> v.getReleaseType())
                            .collect(Collectors.toSet());
            Set<Integer> takeTypeList =
                    createReqVO.getTaskDetailList().stream().map(v -> v.getTakeType())
                            .collect(Collectors.toSet());
            if (releaseTypeList.contains(ReleaseTakeEnum.TO_LOCATION.getType())
                    || takeTypeList.contains(ReleaseTakeEnum.TO_LOCATION.getType())) {
                throw exception(TASK_ONLY_CHOOSE_LOCATION);
            }
        }
        //校验机器人是否禁用
        Set<String> robotNos = createReqVO.getTaskDetailList()
                .stream()
                .map(RobotTaskDetailAddVo::getRobotNo)
                .collect(Collectors.toSet());
        if (ObjectUtil.isNotEmpty(robotNos)) {
            List<RobotInformationDO> robots = informationMapper.selectByRobotNos(robotNos);
            String robotNo = robots.stream()
                    .filter(v -> RobotTaskModelEnum.REJECTION.getType().equals(v.getRobotTaskModel()))
                    .map(RobotInformationDO::getRobotNo)
                    .collect(Collectors.joining(" "));

            if (ObjectUtil.isNotEmpty(robotNo)) {
                throw exception0(TASK_CHECK_EXCEPTION.getCode(), "以下机器人已被禁用 ",robotNo);
            }
        }
    }

    /**
     * 添加任务
     * @param createReqVO
     */
    @Transactional(rollbackFor = Exception.class)
    public String addTask(RobotTaskSaveReqVO createReqVO) {
        RobotTaskDO task = BeanUtils.toBean(createReqVO, RobotTaskDO.class);
        taskMapper.insert(task);

        //查找库位
        if (MoveAllEnum.NO.getType().equals(createReqVO.getDoMoveAll())) {
            setSingleLocationIdNo(createReqVO.getTaskDetailList(),task);
        } else {
            setAllLocationIdNo(createReqVO,task.getId());
        }

        // 插入
        createReqVO.getTaskDetailList()
                .forEach(taskDetail -> {
                    taskDetail.setRobotTaskId(task.getId());
                });
        robotTaskDetailService.insertRobotDetailList(createReqVO.getTaskDetailList());

        return RESULT;
    }

    /**
     * 设置放货和取货的id
     * @param createReqVO
     * @param taskId
     */
    @Transactional(rollbackFor = Exception.class)
    public void setAllLocationIdNo(RobotTaskSaveReqVO createReqVO, Long taskId) {
        List<RobotTaskDetailAddVo> taskDetailList = createReqVO.getTaskDetailList();

        List<RobotTaskDetailAddVo> newTaskDetailList = new ArrayList<>();
        List<Long> toLocationIds = new ArrayList<>(); //被此次任务锁定的空库位id
        List<Long> fromLocationIds = new ArrayList<>(); //被此次任务锁定的有货库位id
        for (RobotTaskDetailAddVo robotTaskVo : taskDetailList) {
            Set<Long> mapIds = new HashSet<>();
            if (ObjectUtil.isNotEmpty(robotTaskVo.getRobotNo())) {
                RobotInformationPageReqVO pageReqVO = new RobotInformationPageReqVO();
                pageReqVO.setRobotNo(robotTaskVo.getRobotNo());
                mapIds = informationMapper.selectPage(pageReqVO).getList()
                        .stream()
                        .findFirst()
                        .map(RobotInformationDO::getFloorAreaJson)
                        .orElse(new HashSet<>());
            }
            //查找有货且非锁定的库位
            WareHouseLocationDO query = null;
            if (ReleaseTakeEnum.TO_LANE.getType().equals(robotTaskVo.getTakeType())) {
                query = WareHouseLocationDO.builder()
                        .laneId(robotTaskVo.getTakeId())
                        .locationEnable(LocationEnableEnum.YES.getType())
                        .locationLock(LocationLockEnum.YES.getType())
                        .locationUseStatus(LocationUseStatusEnum.YES.getType())
                        .build();
            } else if (ReleaseTakeEnum.TO_AREA.getType().equals(robotTaskVo.getTakeType())) {
                query = WareHouseLocationDO.builder()
                        .areaId(robotTaskVo.getTakeId())
                        .locationEnable(LocationEnableEnum.YES.getType())
                        .locationLock(LocationLockEnum.YES.getType())
                        .locationUseStatus(LocationUseStatusEnum.YES.getType())
                        .build();
            }

            List<WareHouseLocationDO> stockList = locationMapper.selectLocations(query,toLocationIds,mapIds);
            if (ObjectUtil.isEmpty(stockList)) {
                log.error("取货线库/区域为空或者已锁定或者机器人取放货区域受限制 :{}",robotTaskVo.toString());
                throw new RuntimeException("取货线库/区域为空或者已锁定或者机器人取放货区域受限制");
            }
            //判断取货库位是否存在未完成的任务
            Set<Long> stockLocationIds = stockList.stream().map(WareHouseLocationDO::getId).collect(Collectors.toSet());
            List<RobotTaskDetailDO> doingTasks = taskDetailMapper.queryDoingTaskByIds(stockLocationIds);
            if (ObjectUtil.isNotEmpty(doingTasks)) {
                throw new RuntimeException("取货线库/区域,存在未完成的任务");
            }
            //查找空库位
            WareHouseLocationDO releaseQuery = null;
            if (ReleaseTakeEnum.TO_LANE.getType().equals(robotTaskVo.getReleaseType())) {
                releaseQuery = WareHouseLocationDO.builder()
                        .laneId(robotTaskVo.getReleaseId())
                        .locationEnable(LocationEnableEnum.YES.getType())
                        .locationLock(LocationLockEnum.YES.getType())
                        .locationUseStatus(LocationUseStatusEnum.NO.getType())
                        .build();
            } else if (ReleaseTakeEnum.TO_AREA.getType().equals(robotTaskVo.getReleaseType())) {
                 releaseQuery = WareHouseLocationDO.builder()
                        .areaId(robotTaskVo.getReleaseId())
                        .locationEnable(LocationEnableEnum.YES.getType())
                        .locationLock(LocationLockEnum.YES.getType())
                        .locationUseStatus(LocationUseStatusEnum.NO.getType())
                        .build();
            }

            List<WareHouseLocationDO> releaseStockList = locationMapper.selectLocations(releaseQuery,toLocationIds,mapIds);
            if (ObjectUtil.isEmpty(releaseStockList) || releaseStockList.size() < stockList.size()) {
                log.error("放货线库/区域库位数量不足或者机器人取放货区域受限制 :{}",robotTaskVo.toString());
                throw new RuntimeException("放货线库/区域库位数量不足或者机器人取放货区域受限制");
            }
            //赋值数据
            List<RobotTaskDetailAddVo> addTaskDetailList = getTaskDatail(stockList,releaseStockList,toLocationIds,
                                                                            robotTaskVo,taskId,fromLocationIds);
            newTaskDetailList.addAll(addTaskDetailList);

        }

        createReqVO.setTaskDetailList(newTaskDetailList);
        //跟新库位状态为锁定
        List<List<Long>> toPartition = Lists.partition(toLocationIds, 100);
        toPartition.forEach(list-> {
            locationMapper.updateLocationLockList(list,taskId);
        });

        List<List<Long>> fromPartition = Lists.partition(fromLocationIds, 100);
        fromPartition.forEach(list-> {
            locationMapper.updateLocationLockList(list,taskId);
        });

    }

    /**
     * 赋值数据
     * @param stockList
     * @param releaseStockList
     * @param toLocationIds
     * @param robotTaskVo
     * @return
     */
    private List<RobotTaskDetailAddVo> getTaskDatail(List<WareHouseLocationDO> stockList,
                                                     List<WareHouseLocationDO> releaseStockList,
                                                     List<Long> toLocationIds, RobotTaskDetailAddVo robotTaskVo,
                                                     Long taskId, List<Long> fromLocationIds) {
        List<RobotTaskDetailAddVo> addTaskDetailList = new ArrayList<>();
        for (int i = 0; i < stockList.size(); i++) {
            RobotTaskDetailAddVo robotTaskDetailAddVo = new RobotTaskDetailAddVo();
            robotTaskDetailAddVo.setTaskType(robotTaskVo.getTaskType());
            robotTaskDetailAddVo.setReleaseType(robotTaskVo.getReleaseType());
            robotTaskDetailAddVo.setTakeType(robotTaskVo.getTakeType());
            robotTaskDetailAddVo.setTakeId(robotTaskVo.getTakeId());
            robotTaskDetailAddVo.setReleaseId(robotTaskVo.getReleaseId());
            robotTaskDetailAddVo.setRobotNo(robotTaskVo.getRobotNo());
            robotTaskDetailAddVo.setFromLocationNo(stockList.get(i).getLocationNo());
            robotTaskDetailAddVo.setFromLocationId(stockList.get(i).getId());
            robotTaskDetailAddVo.setToLocationNo(releaseStockList.get(i).getLocationNo());
            robotTaskDetailAddVo.setToLocationId(releaseStockList.get(i).getId());
            robotTaskDetailAddVo.setRobotTaskId(taskId);
            robotTaskDetailAddVo.setPriority(robotTaskVo.getPriority());
            robotTaskDetailAddVo.setFromLocationStorey(stockList.get(i).getLocationStorey());
            robotTaskDetailAddVo.setToLocationStorey(releaseStockList.get(i).getLocationStorey());
            addTaskDetailList.add(robotTaskDetailAddVo);

            toLocationIds.add(releaseStockList.get(i).getId());
            fromLocationIds.add(stockList.get(i).getId());
        }
        return addTaskDetailList;
    }

    /**
     * 设置放货和取货的id
     * @param taskDetailList
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void setSingleLocationIdNo(List<RobotTaskDetailAddVo> taskDetailList, RobotTaskDO task){
        List<Long> locationIds = new ArrayList<>();
        for (RobotTaskDetailAddVo robotTaskVo : taskDetailList) {
            robotTaskVo.setRobotTaskId(task.getId());
            robotTaskVo.setPriority(task.getPriority());
            RobotTaskTypeEnum robotTaskType = RobotTaskTypeEnum.getRobotTaskType(robotTaskVo.getTaskType());
            switch (robotTaskType){
                case TAKE_RELEASE:
                    doTakeRelease(robotTaskVo,locationIds);
                    break;
                case PARK:
                    doPark(robotTaskVo,locationIds);
                    break;
                case CHARGE:
                    doCharge(robotTaskVo);
                    break;
                case MOVE:
                    doMove(robotTaskVo,locationIds);
                    break;
                case TAKE:
                    doTake(robotTaskVo,locationIds);
                    break;
                case RELEASE:
                    doRelease(robotTaskVo,locationIds);
                    break;
                case SCAN:
                    doScan(robotTaskVo);
                    break;
                case DETECTING_TRAYS:
                    doDetectingTrays(robotTaskVo);
                    break;
                default :
                    log.error("任务类型不存在 ");
                    throw new RuntimeException("任务类型不存在");
            }
        }
    }

    /**
     * 检测托盘类型
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doDetectingTrays(RobotTaskDetailAddVo robotTaskVo) {

    }

    /**
     * 扫描码
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doScan(RobotTaskDetailAddVo robotTaskVo) {

    }

    /**
     * 仅放货
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doRelease(RobotTaskDetailAddVo robotTaskVo, List<Long> locationIds) {
        setToLocation(robotTaskVo,getMapIdsByRobotNo(robotTaskVo.getRobotNo()),locationIds);
        locationMapper.updateLocationLockStatus(robotTaskVo.getToLocationId(),0,robotTaskVo.getRobotTaskId());
        locationIds.add(robotTaskVo.getToLocationId());
    }

    /**
     * 仅取货
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doTake(RobotTaskDetailAddVo robotTaskVo, List<Long> locationIds) {
        locationMapper.updateLocationLockStatus(robotTaskVo.getFromLocationId(),0,robotTaskVo.getRobotTaskId());
        locationIds.add(robotTaskVo.getFromLocationId());
    }

    /**
     * 移动
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doMove(RobotTaskDetailAddVo robotTaskVo, List<Long> locationIds) {
        setToLocation(robotTaskVo,getMapIdsByRobotNo(robotTaskVo.getRobotNo()),locationIds);
        locationIds.add(robotTaskVo.getToLocationId());
    }

    /**
     * 充电
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doCharge(RobotTaskDetailAddVo robotTaskVo) {

    }

    /**
     * 停车
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doPark(RobotTaskDetailAddVo robotTaskVo, List<Long> locationIds) {
        setToLocation(robotTaskVo,getMapIdsByRobotNo(robotTaskVo.getRobotNo()),locationIds);
        locationIds.add(robotTaskVo.getToLocationId());
    }

    /**
     * 查询此机器人对应的楼层/区域(json)
     * @param robotNo
     * @return
     */
    public Set<Long> getMapIdsByRobotNo(String robotNo) {
        if (ObjectUtil.isNotEmpty(robotNo)) {
            RobotInformationPageReqVO pageReqVO = new RobotInformationPageReqVO();
            pageReqVO.setRobotNo(robotNo);
            return  informationMapper.selectPage(pageReqVO).getList()
                    .stream()
                    .findFirst()
                    .map(RobotInformationDO::getFloorAreaJson)
                    .orElse(new HashSet<>());
        }
        return new HashSet<>();
    }

    /**
     * 设置停车/目标库位
     */
    @Transactional(rollbackFor = Exception.class)
    public void setToLocation(RobotTaskDetailAddVo robotTaskVo,Set<Long> mapIds,List<Long> locationIds){
        if (ReleaseTakeEnum.TO_LOCATION.getType().equals(robotTaskVo.getReleaseType())) {
            WareHouseLocationDO query = WareHouseLocationDO.builder().id(robotTaskVo.getReleaseId()).build();
            WareHouseLocationDO locationDO = locationMapper.queryAllByLimit(query);
            robotTaskVo.setToLocationNo(locationDO.getLocationNo());
            robotTaskVo.setToLocationId(robotTaskVo.getReleaseId());
            robotTaskVo.setToLocationStorey(locationDO.getLocationStorey());
            if (ObjectUtil.isNotEmpty(mapIds) && !mapIds.contains(locationDO.getMapId())) {
                log.error("机器人不能在此放货库位放货 :{}, 机器人编号 :{}", locationDO.getLocationNo(),robotTaskVo.getRobotNo());
                throw new RuntimeException("机器人不能在此放货库位放货 "+ robotTaskVo.getReleaseId());
            }
        } else {
            WareHouseLocationDO wareHouseLocationDO =
                    locationMapper.selectByTypeAndId(LocationUseStatusEnum.NO.getType(), robotTaskVo.getReleaseType()
                            ,robotTaskVo.getReleaseId(),locationIds,mapIds);
            if (ObjectUtil.isEmpty(wareHouseLocationDO)) {
                log.error("放货库位为空或者已锁定或者机器人取放货区域受限制 :{}", robotTaskVo.getReleaseId());
                throw new RuntimeException("放货库位为空或者已锁定或者机器人取放货区域受限制 "+ robotTaskVo.getReleaseId());
            }
            robotTaskVo.setToLocationNo(wareHouseLocationDO.getLocationNo());
            robotTaskVo.setToLocationId(wareHouseLocationDO.getId());
            robotTaskVo.setToLocationStorey(wareHouseLocationDO.getLocationStorey());
        }
    }

    /**
     * 设置取货库位
     * @param robotTaskVo
     * @param mapIds
     * @param locationIds
     */
    @Transactional(rollbackFor = Exception.class)
    public void setFromLocation(RobotTaskDetailAddVo robotTaskVo,Set<Long> mapIds,List<Long> locationIds) {
        if (ReleaseTakeEnum.TO_LOCATION.getType().equals(robotTaskVo.getTakeType())) {
            WareHouseLocationDO query = WareHouseLocationDO.builder().id(robotTaskVo.getTakeId()).build();
            WareHouseLocationDO locationDO = locationMapper.queryAllByLimit(query);
            robotTaskVo.setFromLocationNo(locationDO.getLocationNo());
            robotTaskVo.setFromLocationId(robotTaskVo.getTakeId());
            robotTaskVo.setFromLocationStorey(locationDO.getLocationStorey());
            if (ObjectUtil.isNotEmpty(mapIds) && !mapIds.contains(locationDO.getMapId())) {
                log.error("机器人不能在此取货库位取货 :{}, 机器人编号 :{}", locationDO.getLocationNo(),robotTaskVo.getRobotNo());
                throw new RuntimeException("机器人不能在此取货库位取货 "+ robotTaskVo.getTakeId());
            }
        } else {
            WareHouseLocationDO wareHouseLocationDO =
                    locationMapper.selectByTypeAndId(LocationUseStatusEnum.YES.getType(),robotTaskVo.getTakeType(),
                            robotTaskVo.getTakeId(),locationIds,mapIds);
            if (ObjectUtil.isEmpty(wareHouseLocationDO)) {
                log.error("取货库位为空或者已锁定或者机器人取放货区域受限制 :{}", robotTaskVo.getTakeId());
                throw new RuntimeException("取货库位为空或者已锁定或者机器人取放货区域受限制 "+ robotTaskVo.getTakeId());
            }
            robotTaskVo.setFromLocationNo(wareHouseLocationDO.getLocationNo());
            robotTaskVo.setFromLocationId(wareHouseLocationDO.getId());
            robotTaskVo.setFromLocationStorey(wareHouseLocationDO.getLocationStorey());
        }
    }

    /**
     * 取放
     * @param robotTaskVo
     */
    @Transactional(rollbackFor = Exception.class)
    public void doTakeRelease(RobotTaskDetailAddVo robotTaskVo,List<Long> locationIds) {
        Set<Long> mapIds = new HashSet<>();
        if (ObjectUtil.isNotEmpty(robotTaskVo.getRobotNo())) {
            mapIds = getMapIdsByRobotNo(robotTaskVo.getRobotNo());
        }

        //校验放货库位
        setToLocation(robotTaskVo, mapIds,locationIds);
        locationIds.add(robotTaskVo.getToLocationId());

        //校验取货库位
        setFromLocation(robotTaskVo,mapIds,locationIds);
        locationIds.add(robotTaskVo.getFromLocationId());

        //验证取货库位是否存在未完成的任务
        Set<Long> set  = new HashSet<>();
        set.add(robotTaskVo.getFromLocationId());
        List<RobotTaskDetailDO> doingTasks = taskDetailMapper.queryDoingTaskByIds(set);
        if (ObjectUtil.isNotEmpty(doingTasks)) {
            log.error("取货库位,存在未完成的任务 :{}", robotTaskVo.getTakeId());
            throw new RuntimeException("取货库位,存在未完成的任务");
        }

        //设置为锁定
        locationMapper.updateLocationLockStatus(robotTaskVo.getToLocationId(),0,robotTaskVo.getRobotTaskId());
        locationMapper.updateLocationLockStatus(robotTaskVo.getFromLocationId(),0,robotTaskVo.getRobotTaskId());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTask(RobotTaskSaveReqVO updateReqVO) {
        RLock lock = redissonUtils.getLock(RobotCacheLockEnum.ROBOT_TASK_DISTRIBUTE_LOCK.getKey());
        if (lock.tryLock()){
            try {
                // 校验存在
                validateTaskExists(updateReqVO.getId());
                if (ObjectUtil.isEmpty(updateReqVO.getId())) {
                    throw exception(TASK_CHECK_ID_EXCEPTION);
                }
                // 更新
                //修改优先级需要订单状态为未开始
                RobotTaskDO robotTaskDO = taskMapper.selectById(updateReqVO.getId());
                if (ObjectUtil.isNotEmpty(updateReqVO.getPriority())) {
                    if (!RobotTaskStatusEnum.NEW.getType().equals(robotTaskDO.getTaskStatus())) {
                        throw exception(TASK_CHECK_TASK_PRIORITY);
                    }
                    List<RobotTaskDetailDO> taskDetailDOS = taskDetailMapper.queryByTaskId(updateReqVO.getId());
                    for (RobotTaskDetailDO taskDetailDO : taskDetailDOS) {
                        taskDetailDO.setPriority(updateReqVO.getPriority());
                        taskDetailMapper.updateRobotDetailById(taskDetailDO);
                    }
                }
                //修改订单状态
                if (ObjectUtil.isNotEmpty(updateReqVO.getTaskStatus()) &&
                        RobotTaskStatusEnum.DONE.getType().equals(robotTaskDO.getTaskStatus())) {
                    throw exception(TASK_CHECK_TASK_STATUS);
                }
                RobotTaskDO updateObj = BeanUtils.toBean(updateReqVO, RobotTaskDO.class);
                taskMapper.updateById(updateObj);
            } catch (Exception e) {
                log.error("下发任务给车机出现异常 :{}",e.getMessage());
            } finally {
                lock.unlock();
            }
        } else {
            throw exception(REDISSON_NOT_OBTAIN_LOCK);
        }
    }

    @Override
    public void deleteTask(Long id) {
        // 校验存在
        validateTaskExists(id);
        // 删除
        taskMapper.deleteById(id);
    }

    private void validateTaskExists(Long id) {
        if (taskMapper.selectById(id) == null) {
            throw exception(TASK_NOT_EXISTS);
        }
    }

    @Override
    public RobotTaskDO getTask(Long id) {
        return taskMapper.selectById(id);
    }

    @Override
    public PageResult<RobotTaskRespVO> getTaskPage(RobotTaskPageReqVO pageReqVO) {
        PageResult<RobotTaskDO> pageResult = taskMapper.selectPage(pageReqVO);
        PageResult<RobotTaskRespVO> dataPage = new PageResult<>();
        dataPage.setTotal(pageResult.getTotal());
        List<RobotTaskDO> list = pageResult.getList();
        List<RobotTaskRespVO> targetList = BeanUtil.copyToList(list, RobotTaskRespVO.class);
        for (RobotTaskRespVO robotTaskRespVO : targetList) {
            robotTaskRespVO.setDetails(taskDetailMapper.queryByTaskId(robotTaskRespVO.getId()));
        }
        dataPage.setList(targetList);
        return dataPage;
    }

    @Override
    public String getTaskNo() {
        String incrementByKey = redisUtil.getIncrementByKey(RobotCacheLockEnum.TASK_NO.getKey());
        return taskNo+ DateUtils.getYearMonthDay()+incrementByKey;
    }

}